# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-01-11 02:08
from __future__ import unicode_literals

from django.db import migrations, models

try:
    import jsonfield.fields
    field = jsonfield.fields.JSONField()
except ImportError:
    field = models.TextField()


class Migration(migrations.Migration):

    dependencies = [
        ('eventlog', '0002_auto_20150113_1450'),
    ]

    operations = [
        migrations.AlterField(
            model_name='log',
            name='extra',
            field=field,
        ),
    ]



from django.conf import settings
from django.contrib.postgres.fields import JSONField
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.core.serializers.json import DjangoJSONEncoder
from django.db import models
from django.utils import timezone

from .signals import event_logged


class Log(models.Model):

    user = models.ForeignKey(
        getattr(settings, "AUTH_USER_MODEL", "auth.User"),
        null=True,
        on_delete=models.SET_NULL
    )
    timestamp = models.DateTimeField(default=timezone.now, db_index=True)
    action = models.CharField(max_length=50, db_index=True)
    content_type = models.ForeignKey(ContentType, null=True, on_delete=models.SET_NULL)
    object_id = models.PositiveIntegerField(null=True)
    obj = GenericForeignKey("content_type", "object_id")
    extra = JSONField(encoder=DjangoJSONEncoder)

    @property
    def template_fragment_name(self):
        return "pinax/eventlog/{}.html".format(self.action.lower())

    class Meta:
        ordering = ["-timestamp"]


def log(user, action, extra=None, obj=None, dateof=None):
    if (user is not None and not user.is_authenticated):
        user = None
    if extra is None:
        extra = {}
    content_type = None
    object_id = None
    if obj is not None:
        content_type = ContentType.objects.get_for_model(obj)
        object_id = obj.pk
    if dateof is None:
        dateof = timezone.now()

    event = Log.objects.create(
        user=user,
        action=action,
        extra=extra,
        content_type=content_type,
        object_id=object_id,
        timestamp=dateof
    )
    event_logged.send(sender=Log, event=event)
    return event



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models

try:
    import jsonfield.fields
    field = jsonfield.fields.JSONField(default=dict)
except ImportError:
    field = models.TextField()

class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Log',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('timestamp', models.DateTimeField(default=django.utils.timezone.now, db_index=True)),
                ('action', models.CharField(max_length=50, db_index=True)),
                ('extra', field),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'ordering': ['-timestamp'],
            },
            bases=(models.Model,),
        ),
    ]



from .models import log


class EventLogMixin(object):

    @property
    def action(self):
        return "{}_{}".format(
            self.action_kind,
            self.model._meta.verbose_name.upper().replace(" ", "_")
        )

    @property
    def extra_data(self):
        return {}

    @property
    def user(self):
        if self.request.user.is_authenticated:
            return self.request.user
        return None

    def log_action(self):
        log(
            user=self.user,
            action=self.action,
            extra=self.extra_data,
            obj=getattr(self, "object", None)
        )

    def form_valid(self, form):
        response = super(EventLogMixin, self).form_valid(form)
        self.log_action()
        return response



from django.contrib import admin

from .models import Log


class LogAdmin(admin.ModelAdmin):

    raw_id_fields = ["user"]
    list_filter = ["action", "timestamp"]
    list_display = ["timestamp", "user", "action", "extra"]
    search_fields = ["user__username", "user__email", "extra"]


admin.site.register(Log, LogAdmin)



# Generated by Django 2.2.12 on 2020-04-28 22:08

import django.contrib.postgres.fields.jsonb
import django.core.serializers.json
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('eventlog', '0004_auto_20191205_2033'),
    ]

    operations = [
        migrations.AlterField(
            model_name='log',
            name='extra',
            field=django.contrib.postgres.fields.jsonb.JSONField(encoder=django.core.serializers.json.DjangoJSONEncoder),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
        ('eventlog', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='log',
            name='content_type',
            field=models.ForeignKey(to='contenttypes.ContentType', null=True, on_delete=models.SET_NULL),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='log',
            name='object_id',
            field=models.PositiveIntegerField(null=True),
            preserve_default=True,
        ),
    ]



# Generated by Django 3.0 on 2019-12-05 20:33

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('eventlog', '0003_auto_20160111_0208'),
    ]

    operations = [
        migrations.AlterField(
            model_name='log',
            name='extra',
            field=django.contrib.postgres.fields.jsonb.JSONField(),
        ),
    ]



from datetime import datetime, timedelta

from django.contrib.auth import get_user_model


def used_active(days):
    User = get_user_model()
    used = User.objects.filter(
        log__timestamp__gt=datetime.now() - timedelta(days=days)
    ).distinct().count()

    active = User.objects.filter(
        log__timestamp__gt=datetime.now() - timedelta(days=days)
    ).exclude(
        date_joined__gt=datetime.now() - timedelta(days=days)
    ).distinct().count()

    return used, active


def stats():
    used_seven, active_seven = used_active(7)
    used_thirty, active_thirty = used_active(30)

    return {
        "used_seven": used_seven,
        "used_thirty": used_thirty,
        "active_seven": active_seven,
        "active_thirty": active_thirty
    }



from setuptools import find_packages, setup

VERSION = "3.0.1"
LONG_DESCRIPTION = """
.. image:: http://pinaxproject.com/pinax-design/patches/pinax-eventlog.svg
    :target: https://pypi.python.org/pypi/pinax-eventlog/

==============
Pinax EventLog
==============

.. image:: https://img.shields.io/pypi/v/pinax-eventlog.svg
    :target: https://pypi.python.org/pypi/pinax-eventlog/

\

.. image:: https://img.shields.io/circleci/project/github/pinax/pinax-eventlog.svg
    :target: https://circleci.com/gh/pinax/pinax-eventlog
.. image:: https://img.shields.io/codecov/c/github/pinax/pinax-eventlog.svg
    :target: https://codecov.io/gh/pinax/pinax-eventlog
.. image:: https://img.shields.io/github/contributors/pinax/pinax-eventlog.svg
    :target: https://github.com/pinax/pinax-eventlog/graphs/contributors
.. image:: https://img.shields.io/github/issues-pr/pinax/pinax-eventlog.svg
    :target: https://github.com/pinax/pinax-eventlog/pulls
.. image:: https://img.shields.io/github/issues-pr-closed/pinax/pinax-eventlog.svg
    :target: https://github.com/pinax/pinax-eventlog/pulls?q=is%3Apr+is%3Aclosed

\

.. image:: http://slack.pinaxproject.com/badge.svg
    :target: http://slack.pinaxproject.com/
.. image:: https://img.shields.io/badge/license-MIT-blue.svg
    :target: https://opensource.org/licenses/MIT/

\

``pinax-eventlog`` provides an easy and clean interface for logging diagnostic
as well as business intelligence data about activity that occurs in your site.

"""

setup(
    author="Pinax Team",
    author_email="team@pinaxproject.com",
    description="an event logger for Django projects",
    name="pinax-eventlog",
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/x-rst",
    version=VERSION,
    url="http://github.com/pinax/pinax-eventlog/",
    license="MIT",
    packages=find_packages(),
    package_data={
        "eventlog": []
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    install_requires=[
        "django>=2.2",
        "psycopg2-binary>=2.8.4"
    ],
    tests_require=[
    ],
    test_suite="runtests.runtests",
    zip_safe=False
)



from datetime import timedelta

from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase
from django.utils import timezone

from ..mixins import EventLogMixin
from ..models import Log, log
from ..signals import event_logged
from ..stats import stats, used_active


class TestStats(TestCase):

    def setUp(self):
        user = get_user_model().objects.create(
            date_joined=(timezone.now() - timedelta(days=100))
        )
        log(action="SOME_EVENT", user=user, extra={})

    def test_used_active(self):
        used, active = used_active(10)
        self.assertEquals(used, 1)
        self.assertEquals(active, 1)

    def test_stats(self):
        data = stats()
        self.assertEquals(data, {
            "used_seven": 1,
            "used_thirty": 1,
            "active_seven": 1,
            "active_thirty": 1
        })


class TestModels(TestCase):

    def test_template_fragment_name(self):
        event = log(user=None, action="FOO")
        self.assertEquals(event.template_fragment_name, "pinax/eventlog/foo.html")

    def test_log_user_not_authed(self):
        user = AnonymousUser()
        event = log(user=user, action="FOO", extra={})
        self.assertIsNone(event.user)

    def test_log_extra_not_provided(self):
        event = log(user=None, action="FOO")
        self.assertEquals(event.extra, {})

    def test_log_content_object(self):
        user = get_user_model().objects.create(
            date_joined=(timezone.now() - timedelta(days=100))
        )
        event = log(user=None, action="FOO", obj=user)
        self.assertEquals(event.object_id, user.pk)
        self.assertEquals(event.content_type, ContentType.objects.get_for_model(user))

    def test_log_dateof(self):
        dateof = timezone.now() - timedelta(days=3)
        event = log(user=None, action="FOO", dateof=dateof)
        self.assertEquals(event.timestamp, dateof)

    def test_signal_sent(self):
        def _handler(sender, event, **kwargs):
            self.assertEquals(event.action, "FOO_SENT")
            event.sent = True

        event_logged.connect(_handler)
        event = log(user=None, action="FOO_SENT")
        self.assertTrue(event.sent)


class RequestMock(object):
    def __init__(self, user):
        self.user = user


class TestMixins(TestCase):

    def test_action(self):
        mixin = EventLogMixin()
        mixin.action_kind = "CREATE"
        mixin.model = get_user_model()
        self.assertEquals(mixin.action, "CREATE_USER")

    def test_extra_data(self):
        self.assertEquals(EventLogMixin().extra_data, {})

    def test_user(self):
        user = get_user_model().objects.create(
            date_joined=(timezone.now() - timedelta(days=100))
        )
        mixin = EventLogMixin()
        mixin.request = RequestMock(user)
        self.assertEquals(mixin.user, user)

    def test_log_action(self):
        user = get_user_model().objects.create(
            date_joined=(timezone.now() - timedelta(days=100))
        )
        mixin = EventLogMixin()
        mixin.action_kind = "CREATE"
        mixin.model = get_user_model()
        mixin.request = RequestMock(user)
        mixin.log_action()
        self.assertEquals(Log.objects.all()[0].action, "CREATE_USER")
